#ifndef __TASK_HPP__
#define __TASK_HPP__

#include <coroutine.hpp>
#include <result.hpp>


namespace typon
{

  template <typename T = void>
  struct [[nodiscard]] Task
  {
    struct promise_type;

    coroutine_handle<promise_type> _coroutine;

    ~Task()
    {
      _coroutine.destroy();
    }

    struct promise_type : Result<T>
    {
      coroutine_handle<> _continuation;

      Task get_return_object() noexcept
      {
        return { coroutine_handle<promise_type>::from_promise(*this) };
      }

      suspend_always initial_suspend() noexcept
      {
        return {};
      }

      auto final_suspend() noexcept
      {
        struct awaitable : suspend_always
        {
          coroutine_handle<> await_suspend(coroutine_handle<promise_type> coroutine) noexcept
          {
            return coroutine.promise()._continuation;
          }
        };

        return awaitable {};
      }
    };

    auto operator co_await() && noexcept
    {
      struct awaitable
      {
        coroutine_handle<promise_type> _coroutine;

        bool await_ready() noexcept
        {
          return false;
        }

        coroutine_handle<> await_suspend(coroutine_handle<> awaiting_coroutine) noexcept
        {
          _coroutine.promise()._continuation = awaiting_coroutine;
          return _coroutine;
        }

        decltype(auto) await_resume()
        {
          return _coroutine.promise().get();
        }
      };

      return awaitable { _coroutine };
    }

    auto discard() && noexcept
    {
      struct awaitable : suspend_always
      {
        coroutine_handle<promise_type> _coroutine;

        coroutine_handle<> await_suspend(coroutine_handle<> awaiting_coroutine) noexcept
        {
          _coroutine.promise()._continuation = awaiting_coroutine;
          return _coroutine;
        }
      };

      return awaitable { {}, _coroutine };
    }

    decltype(auto) call() &&
    {
      _coroutine.promise()._continuation = noop_coroutine();
      _coroutine.resume();
      return _coroutine.promise().get();
    }

    void call_discard() &&
    {
      _coroutine.promise()._continuation = noop_coroutine();
      _coroutine.resume();
    }
  };
  
}

#endif // __TASK_HPP__
